#include "effects.h"
#include "global.h"

SDL_Surface *font_bmp=0;
static SDL_Surface *scroller0=0;
static SDL_Surface *scroller1=0;
static SDL_Surface *scroller2=0;
static int *star=0;
static const int starNum=200;

static void cleanup()
{
  if(font_bmp)
    SDL_FreeSurface(font_bmp), font_bmp=0;
  if(scroller0)
    SDL_FreeSurface(scroller0), scroller0=0;
  if(scroller1)
    SDL_FreeSurface(scroller1), scroller1=0;
  if(scroller2)
    SDL_FreeSurface(scroller2), scroller2=0;
  if(star)
    delete [] star, star=0;
}

int init_scroller()
{
  atexit(cleanup);
  if(!(font_bmp=SDL_LoadBMP("font.bmp")))
    return -1;
  SDL_Color *pal=font_bmp->format->palette->colors;
  pal[0].r=pal[0].g=pal[0].b=0;
  pal[1].r=pal[1].g=pal[1].b=64;
  pal[2].r=pal[2].g=pal[2].b=128;
  pal[3].r=pal[3].g=pal[3].b=160;
  pal[4].r=pal[4].g=pal[4].b=200;
  pal[5].r=pal[5].g=pal[5].b=255;

  // starfield
  if(!(star=new int[starNum*4*2]))
    return -1;
  for(int i=0; i<starNum*4; i++)
  {
    star[2*i]=rand()%320;
    star[2*i+1]=rand()%200;
  }

  // scrollers
  if(!(scroller0=makescroller("                   yeah, what a nice starfield. and it has this certain 3d look....  cool, well i wonder if you could make a starfield and a scroller together. lets give it a try, because that would clearly rule everything.")))
    return -1;
  if(!(scroller1=makescroller("                 yo cool. it is possible i think you never thought i could do it.                                                                yes it is unbelievable i bet you have never seen anzthing like this in your hole live before. well, because there is always room for improvement you can expect us to show you much more stunning effects in this first megademonstration by topcrew. you see, we want to become the top demo crew of the whole world and we keep trying very hard to increase our coding. yes we like this music very much....")))
    return -1;
  if(!(scroller2=makescroller("                                                                                             oh, whats that   a second scroller....              yes it is unbelievable two scrollers at the same time !!!! only topcrew can do such good thing. if you never have heard anything of us before thats no wonder, because we have never released anything yet, but we plan to continue to write good demos so you can expect us to release more of cool demos in the future. anyway we hope you like the music. it was specially composed by wavebrain for the demo.")))
    return -1;

  return 0;
}

SDL_Surface* makescroller(char *intext)
{
  const int textlen=strlen(intext);
  SDL_Surface *s=0;
  if(!(s=SDL_CreateRGBSurface(SDL_HWSURFACE|SDL_SRCCOLORKEY, textlen*16, 36, 8, 0, 0, 0, 0)))
    return 0;
  SDL_SetColors(s, font_bmp->format->palette->colors, 0, 256);
  SDL_SetColorKey(s, SDL_SRCCOLORKEY, 255);

  char *text=maketext(intext);
  if(!text)
  {
    SDL_FreeSurface(s);
    return 0;
  }

  for(int i=0; i<textlen; i++)
  {
    char chr=text[i];
    int fy=0;
    while(chr>18)
    {
      fy+=36;
      chr-=19;
    }
    SDL_Rect src={chr*16, fy, 16, 36};
    SDL_Rect dst={i*16,       0, 16, 36};
    SDL_BlitSurface(font_bmp, &src, s, &dst);
  }

  delete [] text;
  return s;
}

inline void drawpoint(int x, int y, unsigned char color, unsigned char *scr)
{
  if(x>=0 && x<320)
    if(y>=0 && y<200)
      *(scr+320*y+x)=color;
}

void drawline(int x1, int y1, int x2, int y2, unsigned char color, unsigned char *scr)
{
  int i;

  if(x2<x1)
    i=x1, x1=x2, x2=i;
  if(y2<y1)
    i=y1, y1=y1, y2=i;

  const int xd=x2-x1;
  const int yd=y2-y1;

  if(xd==0)         // Senkrechte Linie
    for(i=y1; i<=y2; i++)
      drawpoint(x1,i,color,scr);

  else if(yd==0)    // horizontal
    for(i=x1; i<=x2; i++)
      drawpoint(i,y1,color,scr);

  else
  {
    const double s=(double)yd/(double)xd;

    i=0;
    if(s>=-1 && s<=1)
    {
      const int add=(xd>=0)?1:-1;
      while(i!=xd)
      {
        drawpoint(x1+i,(int)(s*i)+y1,color,scr);
        i+=add;
      }
    }
    else
    {
      const int add=(yd>=0)?1:-1;
      while(i!=yd)
      {
        drawpoint((int)((double)i/s)+x1,y1+i,color,scr);
        i+=add;
      }
    }
  }
}

void pal_rot(SDL_Color *pal, int anfang, int ende)
{
  SDL_Color t=pal[anfang];
  for(int i=anfang; i<ende; i++)
    pal[i]=pal[i+1];
  pal[ende]=t;
}

void pal_make(SDL_Color *pal, int anfang, int ende, unsigned char r1, unsigned char g1, unsigned char b1, unsigned char r2, unsigned char g2, unsigned char b2)
{
  const int steps=ende-anfang;
  const double rd=((double)r2-(double)r1)/steps;
  const double gd=((double)g2-(double)g1)/steps;
  const double bd=((double)b2-(double)b1)/steps;

  for(int i=0; i<=steps; i++)
  {
    pal[anfang+i].r=static_cast<unsigned char>(r1+rd*i);
    pal[anfang+i].g=static_cast<unsigned char>(g1+gd*i);
    pal[anfang+i].b=static_cast<unsigned char>(b1+bd*i);
  }
}

void windowslines(int modbegin)
{
  SDL_Color pal[256];

  unsigned char currentcolor=0;
  int add[4];
  int pos[4];

  int i;
  int flag=0;
  do {

    if(getrow()==0)
      switch(flag%4)
      {
        case 0: fillscreen(0, false);
                pal_make(pal,0,255,0,0,0,63,63,63);
                flag++;
                currentcolor=255;
                pos[0]=10; pos[1]=10; pos[2]=10; pos[3]=10;
                for(i=0; i<4; i++)
                  add[i]=(rand()%5)+1;
                break;
        case 1: fillscreen(0, false);
                pal_make(pal,0,255,0,0,0,63,0,0);
                flag++;
                currentcolor=255;
                pos[0]=10; pos[1]=10; pos[2]=10; pos[3]=10;
                for(i=0; i<4; i++)
                  add[i]=(rand()%5)+1;
                break;
        case 2: fillscreen(0, false);
                pal_make(pal,0,255,0,0,0,0,63,0);
                flag++;
                currentcolor=255;
                pos[0]=10; pos[1]=10; pos[2]=10; pos[3]=10;
                for(i=0; i<4; i++)
                  add[i]=(rand()%5)+1;
                break;
        case 3: fillscreen(0, false);
                pal_make(pal,0,255,0,0,0,0,0,63);
                flag++;
                currentcolor=255;
                pos[0]=10; pos[1]=10; pos[2]=10; pos[3]=10;
                for(i=0; i<4; i++)
                  add[i]=(rand()%5)+1;
                break;
      }

    pal_rot(pal,1,255);

    for(i=0; i<4; i++)
      pos[i]+=add[i];

    for(i=0; i<4; i++)
      if(pos[i]<10)
      {
        pos[i]=10;
        add[i]=(rand()%5)+1;
      }
    if(pos[0]>309)
    {
      pos[0]=309;
      add[0]=-((rand()%5)+1);
    }
    if(pos[2]>309)
    {
      pos[2]=309;
      add[2]=-((rand()%5)+1);
    }
    if(pos[1]>189)
    {
      pos[1]=189;
      add[1]=-((rand()%5)+1);
    }
    if(pos[3]>189)
    {
      pos[3]=189;
      add[3]=-((rand()%5)+1);
    }

    SDL_LockSurface(screen);
    drawline(pos[0],pos[1],pos[2],pos[3],currentcolor++, reinterpret_cast<unsigned char*>(screen->pixels));
    SDL_UnlockSurface(screen);
    SDL_SetColors(screen, pal, 0, 256);
    SDL_UpdateRect(screen);

    checkabort();

  } while(getpos()<modbegin+10);
}

char* maketext(char *in)
{
  char *out=new char[strlen(in)+1];
  if(!out)
    return 0;
  char *ret=out;
  while(*in)
  {
    switch(*in)
    {
    case '!': *out=36; break;
    case '\'':
    case '`': *out=37;
    case '*': *out=38; break;
    case '(': *out=39; break;
    case ')': *out=40; break;
    case '-': *out=41; break;
    case ' ': *out=42; break;
    case '.': *out=43; break;
    case ',': *out=44; break;
    case '/': *out=47; break;
    case ';': *out=49; break;
    case ':': *out=50; break;
    default:
      if(*in>96 && *in<123)      // Buchstaben
        *out=*in-97;
      else if(*in>47 && *in<58)  // Zahlen
        *out=*in-(48-25);
    }
    ++in;
    ++out;
  }
  *out=0;
  return ret;
}

void starfield(unsigned char *scr)
{
  for(int y=0; y<4; y++)
    for(int x=0; x<starNum; x++)
    {
      int currentstar=(y*starNum+x)*2;
      star[currentstar]-=(y+1);
      if(star[currentstar]<0)
      {
        star[currentstar+1]=rand()%200;
        star[currentstar]=319-rand()%4;
      }

      drawpoint(star[currentstar],star[currentstar+1],240+y,scr);
    }
}

void sucking_scroller(int modbegin)
{
  int i;
  Uint32 starttime;
  double time;

  // palette fr Sterne einstellen
  SDL_SetColors(screen, font_bmp->format->palette->colors, 0, 256);

  // 1. Teil
  do {
    SDL_LockSurface(screen);
    memset(screen->pixels, 0, screen->w*screen->h); // make black
    starfield(reinterpret_cast<unsigned char*>(screen->pixels));
    SDL_UnlockSurface(screen);
    SDL_UpdateRect(screen);
    checkabort();
  } while(getpos()<modbegin+1);
  fillscreen(0);

  // 2. Teil
  starttime=SDL_GetTicks();
  do {
    time=static_cast<double>(SDL_GetTicks()-starttime)*.001;
    i=static_cast<int>(time*200.0);

    fillscreen(0, false);
    SDL_Rect src={i, 0,   320, 36};
    SDL_Rect dst={0, 160, 320, 36};

    SDL_BlitSurface(scroller0, &src, screen, &dst);
    SDL_UpdateRect(screen, &dst);
    checkabort();
  } while (getpos()<modbegin+4);

  // 3. Teil
  starttime=SDL_GetTicks();
  do {
    time=static_cast<double>(SDL_GetTicks()-starttime)*.001;
    i=static_cast<int>(time*130.0);

    SDL_LockSurface(screen);
    memset(screen->pixels, 0, screen->w*screen->h); // make black
    starfield(reinterpret_cast<unsigned char*>(screen->pixels));
    SDL_UnlockSurface(screen);

    SDL_Rect src={i, 0, 320, 36};
    SDL_Rect dst={0, (int)(sin(time*20.0*M_PI/180.0)*70.0+85.0), 320, 36};
    SDL_BlitSurface(scroller1, &src, screen, &dst);

    dst.y=(int)(sin(time*20.0*M_PI/180.0+90.0)*70.0+85.0);
    SDL_BlitSurface(scroller2, &src, screen, &dst);

    SDL_UpdateRect(screen);
    checkabort();
  } while(getpos()<modbegin+21);
}
