#include <time.h>

#include <fmod.h>
#include <fmod_errors.h>

#include "global.h"

#include "intro.h"
#include "effects.h"
#include "playani.h"
#include "duck.h"
#include "show.h"
#include "xcrap.h"
#include "pix.h"
#include "lissa.h"

FMUSIC_MODULE *mod=0;

int getpos()
{
  return FMUSIC_GetOrder(mod);
}

int getrow()
{
  return FMUSIC_GetRow(mod);
}

void setpalette(SDL_Surface *s)
{
  if (s->format->palette && screen->format->palette)
    SDL_SetColors(screen, s->format->palette->colors, 0, s->format->palette->ncolors);
}

void blitsurface(SDL_Surface *s)
{
  setpalette(s);
  /* Blit onto the screen surface */
  if(SDL_BlitSurface(s, NULL, screen, NULL) < 0)
      fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());

  SDL_UpdateRect(screen, 0, 0, s->w, s->h);
}

void fillsurface(SDL_Surface *s, int color, bool blit)
{
  SDL_LockSurface(s);
  memset(s->pixels, color, s->w*s->h);
  SDL_UnlockSurface(s);
  if(blit)
    SDL_UpdateRect(s, 0, 0, s->w, s->h);
}

void checkabort()
{
  SDL_Event event;
  if(SDL_PollEvent(&event))
    switch(event.type)
    {
    case SDL_QUIT: exit(0);
    case SDL_KEYDOWN:
        switch(event.key.keysym.sym)
        {
        case SDLK_q:
        case SDLK_ESCAPE:
          exit(0);
	default: ;
        }
    default: ;
    }
}

SDL_Surface *screen=0;
SDL_Surface *endansi=0;

static void cleanup()
{
  if(mod)
  {
    FMUSIC_FreeSong(mod);
    FSOUND_Close();
    mod=0;
  }
  if(screen)
    SDL_FreeSurface(screen), screen=0;
  if(endansi)
    SDL_FreeSurface(endansi), endansi=0;
}

int main(int argc, char *argv[])
{
  srand(time(NULL));
  
  // init SDL
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
  {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);
  atexit(cleanup);

  SDL_WM_SetCaption("Stallinator - ToPCReW", NULL /*const char *icon*/);

  // init SDL graphics
  screen = SDL_SetVideoMode(320, 200, 8, SDL_SWSURFACE
#ifdef WIN32
    |SDL_FULLSCREEN
#endif
    );
  if ( screen == NULL )
  {
    fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
    exit(1);
  }

  // init some graphics
  endansi=SDL_LoadBMP("endansi.bmp");
  if(endansi==NULL)
  {
    fprintf(stderr, "could not load endansi.bmp: %s\n", SDL_GetError());
    exit(1);
  }

  if(init_intro() < 0)
  {
    fprintf(stderr, "could not init intro: %s\n", SDL_GetError());
    exit(1);
  }

  if(init_scroller() < 0)
  {
    fprintf(stderr, "could not init scroller: %s\n", SDL_GetError());
    exit(1);
  }

  if(init_playani() < 0)
  {
    fprintf(stderr, "could not init playani: %s\n", SDL_GetError());
    exit(1);
  }

  if(init_duck() < 0)
  {
    fprintf(stderr, "could not init duck: %s\n", SDL_GetError());
    exit(1);
  }

  if(init_show() < 0)
  {
    fprintf(stderr, "could not init show: %s\n", SDL_GetError());
    exit(1);
  }

  if(init_pix() < 0)
  {
    fprintf(stderr, "could not init pix: %s\n", SDL_GetError());
    exit(1);
  }

  // init fmod
  if (FSOUND_GetVersion() < FMOD_VERSION)
  {
    printf("Error : You are using the wrong DLL version!  You should be using FMOD %.02f\n", FMOD_VERSION);
    exit(1);
  }
  if (!FSOUND_Init(44100, 64, 0))
  {
    printf("%s\n", FMOD_ErrorString(FSOUND_GetError()));
    exit(1);
  }
  mod = FMUSIC_LoadSong("intro.xm");
  if (!mod)
  {
    printf("%s\n", FMOD_ErrorString(FSOUND_GetError()));
    exit(1);
  }

  // start
  FMUSIC_PlaySong(mod);
  //FMUSIC_SetOrder(mod, 5);

  if(getpos()<19)
    intro();
  if(getpos()<0x28)
    sucking_scroller(19);
  if(getpos()<0x32)
    windowslines(0x28);
  if(getpos()<0x3b)
    playani(0x32);
  if(getpos()<0x3f)
    duckeffect(0x3b);
  if(getpos()<0x41)
    show(0x3f);
  if(getpos()<0x45)
    do_crap(0x41);
  if(getpos()<0x49)
    pix(0x45);
  lisa(0x49);

  // endansi
  blitsurface(endansi);  

  while(getpos()>10)
  {
    checkabort();
    SDL_Delay(100);
  }
  return 0;
}
